      SUBROUTINE PODIFF(IDIF1,IDIF2,IMOTH1,IMOTH2,MSOFT,MHARD,
     &                  IMODE,IREJ)
C**********************************************************************
C
C     preparation of common HEPEVS for diffractive chains
C
C     input:   IDIF1  0   quasi elastic scattering of particle 1
C                     1   diffractive dissociation of particle 1
C
C              IDIF2  0   quasi elastic scattering of particle 2
C                     1   diffractive dissociation of particle 2
C
C              IMOTH1/2   index of mother particles in COMMON HEPEVS
C
C              IMODE   0  sampling of diffractive cut
C                      1  sampling of enhanced cut
C                      2  sampling of diffractive cut without
C                         scattering (needed for double-pomeron)
C                     -1  initialization
C                     -2  output of statistics
C
C     output:   MSOFT     number of generated soft chains
C               MHARD     number of generated hard chains
C               IDIF1/2  diffraction label for particle 1/2
C                      0   quasi elastic scattering
C                      1   low-mass diffractive dissociation
C                      2   soft high-mass diffractive dissociation
C                      3   hard resolved diffractive dissociation
C                      4   hard direct diffractive dissociation
C               IREJ   0  successful generation of partons
C                      1  generation failure
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER(EPS  = 1.D-7,
     &          IONE = 1,
     &          ONE  = 1.D0,
     &          TWO  = 2.D0,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          DEPS = 1.D-10,
     &          ZERO = 0.D0)
C
      PARAMETER (NMAXD=100)
      COMMON /PROCES/ IPROCE,IDNODF,IDIFR1,IDIFR2,IDDPOM,IPRON(15)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
C  CM system of diffraction
      COMMON /DIFCMS/ ECMD,PCMD,PMASSD(2),PVIRTD(2),GAMBED(4),
     &                SIDD,CODD,SIFD,COFD,PDCMS(4,2),NPOSD(2)
      COMMON /CUTOFF/ PTCUT(4),CUTMU(4),FPS(4),FPH(4),PSOMIN,XSOMIN
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      PARAMETER (IEETAB=20)
      COMMON /XSETAB/ SIGTAB(4,70,IEETAB),SIGECM(4,IEETAB),ISIMAX
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      DIMENSION P1(5),P2(5),XMASS(2),AMP(2),PP(7,2)
      DIMENSION IDPDG(2),IDBAM(2),IPAR(2),IPOSP(2,2),IRPDG(2),IVEC(2),
     &  IRBAM(2),IFL1P(2),IFL2P(2),ISAM(2),JSAM(2),KSAM(2),LSAM(2),
     &  MSAM(2),IDIR(2)
*     CHARACTER*79 TITLE
*     DIMENSION XLIMB(2)
C
C  initialization/ output of statistics
      IF(IMODE.EQ.-1) THEN
*       CALL NEWHIS(0.D0,15.D0,0.D0,XLIMB,49,IM1)
*       CALL NEWHIS(0.D0,15.D0,0.D0,XLIMB,49,IM2)
        RETURN
      ELSE IF(IMODE.EQ.-2) THEN
*       TITLE = 'M1*DN/DM1  MASS DISTRIBUTION PART.1 DIFFRACTION'
*       CALL OUTHIS(IM1,1,TITLE,1.D0,1)
*       TITLE = 'M2*DN/DM2  MASS DISTRIBUTION PART.2 DIFFRACTION'
*       CALL OUTHIS(IM2,1,TITLE,1.D0,1)
        RETURN
      ENDIF
      IREJ = 0
C  critical/supercritical Pomeron assumed
      ISCP = ISWMDL(22)
C  mass cuts
      PIMASS  = 0.140D0
C  debug output
      IF(IDEB(45).GE.10) THEN
        WRITE(6,'(1X,A,/5X,7I4)')
     &   'PODIFF:DEBUG(1):IDIF1,IDIF2,IMOTH1,IMOTH2,MSOFT,MHARD,IMODE',
     &    IDIF1,IDIF2,IMOTH1,IMOTH2,MSOFT,MHARD,IMODE
      ENDIF
      IPAR(1) = IDIF1
      IPAR(2) = IDIF2
C  range of T
      TMIN = -1D-08
      TMAX = -MAX(ONE/PIMASS**2,6.D0)
C  save chain numbers
      MSOFT = 0
      MHARD = 0
      KHPOMS = KHPOM
      KSPOMS = KSPOM
      KSREGS = KSREG
      KHDIRS = KHDIR
      JDA11 = JDAHEP(1,IMOTH1)
      JDA21 = JDAHEP(2,IMOTH1)
      JDA12 = JDAHEP(1,IMOTH2)
      JDA22 = JDAHEP(2,IMOTH2)
      ISTH1 = ISTHEP(IMOTH1)
      ISTH2 = ISTHEP(IMOTH2)
C  get mother data
      NPOSD(1) = IMOTH1
      NPOSD(2) = IMOTH2
      DO 20 I=1,2
        IDPDG(I) = IDHEP(NPOSD(I))
        IDBAM(I) = IMPART(NPOSD(I))
        AMP(I) = POPAMA(IDBAM(I),0)
        IF(IDPDG(I).EQ.22) THEN
          PMASSD(I) = 0.765D0
          PVIRTD(I) = PHEP(5,NPOSD(I))**2
        ELSE
          PMASSD(I) = POPAMA(IDBAM(I),0)
          PVIRTD(I) = ZERO
        ENDIF
 20   CONTINUE
C  get CM system
      P1(1) = PHEP(1,IMOTH1)+PHEP(1,IMOTH2)
      P1(2) = PHEP(2,IMOTH1)+PHEP(2,IMOTH2)
      P1(3) = PHEP(3,IMOTH1)+PHEP(3,IMOTH2)
      P1(4) = PHEP(4,IMOTH1)+PHEP(4,IMOTH2)
      SS = P1(4)**2-P1(1)**2-P1(2)**2-P1(3)**2
      ECMD = SQRT(SS)
      IF(IDEB(45).GE.15) WRITE(6,'(1X,A,E12.4)')
     &  'PODIFF:DEBUG:AVAILABE ENERGY',ECMD
C  check total available energy
      IF((AMP(1)+AMP(2)+0.1D0).GE.ECMD) THEN
        IFAIL(7) = IFAIL(7)+1
        IREJ = 1
        RETURN
      ENDIF
C  boost into CMS
      DO 10 I=1,4
        GAMBED(I) = P1(I)/ECMD
 10   CONTINUE
      CALL ALTRA(GAMBED(4),-GAMBED(1),-GAMBED(2),-GAMBED(3),
     &           PHEP(1,IMOTH1),PHEP(2,IMOTH1),PHEP(3,IMOTH1),
     &           PHEP(4,IMOTH1),PTOT1,P1(1),P1(2),P1(3),P1(4))
C  rotation angles
      CODD = P1(3)/PTOT1
      SIDD = SQRT((ONE-CODD)*(ONE+CODD))
      COFD = ONE
      SIFD = ZERO
      IF(PTOT1*SIDD.GT.DEPS) THEN
        COFD = P1(1)/(SIDD*PTOT1)
        SIFD = P1(2)/(SIDD*PTOT1)
        ANORF= SQRT(COFD*COFD+SIFD*SIFD)
        COFD = COFD/ANORF
        SIFD = SIFD/ANORF
      ENDIF
C  initial particles in CMS
      PDCMS(1,1) = ZERO
      PDCMS(2,1) = ZERO
      PDCMS(3,1) = PTOT1
      PDCMS(4,1) = P1(4)
      PDCMS(1,2) = ZERO
      PDCMS(2,2) = ZERO
      PDCMS(3,2) = -PTOT1
      PDCMS(4,2) = ECMD-P1(4)
C  get new CM momentum
      AM12 = PMASSD(1)**2
      AM22 = PMASSD(2)**2
      PCMD = XLAMB(SS,AM12,AM22)/(2.D0*ECMD)
C
C  coherence constraint (min/max diffractive mass allowed)
      IF(IMODE.EQ.2) THEN
        THRM1 = PARMDL(71)/SQRT(1-PARMDL(72))
        THRM1 = MAX(THRM1,PARMDL(70)*PARMDL(71))
        THRM2 = SQRT(1-PARMDL(72))*ECMD
        THRM2 = MIN(THRM2,ECMD/PARMDL(70))
      ELSE
        THRM1 = PARMDL(46)
        THRM2 = PARMDL(45)*ECMD
C  check kinematic limits
        IF(THRM2.LE.(4.D0*CUTMU(2))) THEN
          IPAR(1) = MIN(IPAR(1),1)
          IPAR(2) = MIN(IPAR(2),1)
        ENDIF
      ENDIF
C  check energy vs. coherence constraints
      IF(MAX(CUTMU(2),PMASSD(1)+THRM1).GE.THRM2) IPAR(1) = 0
      IF(MAX(CUTMU(3),PMASSD(2)+THRM1).GE.THRM2) IPAR(2) = 0
      IF(IPAR(1)+IPAR(2).EQ.0) THEN
        IFAIL(7) = IFAIL(7)+1
        IREJ = 1
        RETURN
      ENDIF
C
      ITRY = 0
      ITRYM = 10
      NSLP = 0
      NHEPS = NHEP
      IPARS1 = IPAR(1)
      IPARS2 = IPAR(2)
C  main rejection loop
C -------------------------------
 50   CONTINUE
      ITRY = ITRY+1
      IF(ITRY.GT.1) THEN
        IFAIL(13) = IFAIL(13)+1
        IF(ITRY.GE.ITRYM) THEN
          IFAIL(7) = IFAIL(7)+1
          IREJ = 1
          RETURN
        ENDIF
      ENDIF
      KSPOM = KSPOMS
      KHPOM = KHPOMS
      KHDIR = KHDIRS
      KSREG = KSREGS
      IPAR(1) = IPARS1
      IPAR(2) = IPARS2
C  reset mother-daugther relations
      NHEP = NHEPS
      JDAHEP(1,IMOTH1) = JDA11
      JDAHEP(2,IMOTH1) = JDA21
      JDAHEP(1,IMOTH2) = JDA12
      JDAHEP(2,IMOTH2) = JDA22
      ISTHEP(IMOTH1) = ISTH1
      ISTHEP(IMOTH2) = ISTH2
C
C  main loop
      DO 100 I=1,2
C  sampling of new masses
        IRPDG(I) = 0
        IRBAM(I) = 0
        IFL1P(I) = IDPDG(I)
        IFL2P(I) = IDBAM(I)
        IF(IPAR(I).EQ.0) THEN
C  vector meson dominance assumed
          XMASS(I) = AMP(I)
          CALL VECRES(IVEC(I),XMASS(I),IFL1P(I),IFL2P(I))
C  diffractive dissociation
        ELSE IF(IPAR(I).LE.4) THEN
          XMMIN = MAX(CUTMU(I+1),PMASSD(I)+THRM1)
          PREF2 = PMASSD(I)**2+PVIRTD(I)
          XMASS(I) = XDMASS(XMMIN,THRM2,PREF2,ISCP)
          IF(IMODE.EQ.2) GOTO 130
          IP = I+1
C  get cross sections
          IF(XMASS(I).LE.SIGECM(IP,1)) THEN
            IN1 = 1
            IN2 = 1
          ELSE IF(XMASS(I).LT.SIGECM(IP,ISIMAX)) THEN
            DO 55 II=2,ISIMAX
              IF(XMASS(I).LE.SIGECM(IP,II)) GOTO 210
 55         CONTINUE
 210        CONTINUE
            IN1 = II-1
            IN2 = II
          ELSE
            WRITE(6,'(/1X,A,2E12.3)')
     &      'PODIFF:WARNING:TOO HIGH XM',XMASS(I),SIGECM(IP,ISIMAX)
            IN1 = ISIMAX
            IN2 = ISIMAX
          ENDIF
          FAC2=0.D0
          IF(IN1.NE.IN2) FAC2=LOG(XMASS(I)/SIGECM(IP,IN1))
     &                        /LOG(SIGECM(IP,IN2)/SIGECM(IP,IN1))
          FAC1=1.D0-FAC2
          STOT = FAC2*SIGTAB(IP,1,IN2)+FAC1*SIGTAB(IP,1,IN1)
          SELA = FAC2*SIGTAB(IP,2,IN2)+FAC1*SIGTAB(IP,2,IN1)
          SINE = STOT-SELA
          SDIR = FAC2*SIGTAB(IP,29,IN2)+FAC1*SIGTAB(IP,29,IN1)
          SDIR = MAX(ZERO,SDIR)/SINE
C  sample interaction
          IF((RNDM(FAC1).LT.SDIR).AND.(IPRON(8+I).NE.0)) THEN
            IDIR(I) = 1
            ISAM(I) = 0
            JSAM(I) = 0
            KSAM(I) = 0
            LSAM(I) = 0
            MSAM(I) = 0
          ELSE
            CALL SAMPRB(XMASS(I),IP,ISAM(I),JSAM(I),KSAM(I),LSAM(I),
     &        MSAM(I))
          ENDIF
C  suppress hard diffraction
          IF(IPRON(8+I).EQ.0) KSAM(I) = 0
C  suppress multiple interaction
          IF(ISWMDL(16).EQ.0) THEN
            KSAM(I) = 0
            IDIR(I) = 0
            IF(ISAM(I)+KSAM(I).GT.0) THEN
              ISAM(I) = 1
              JSAM(I) = 0
            ELSE
              JSAM(I) = 1
            ENDIF
          ELSE IF(ISWMDL(16).EQ.1) THEN
            IF(IDIR(I).GT.0) THEN
            ELSE IF(KSAM(I).GT.0) THEN
              KSAM(I) = 1
              ISAM(I) = 0
              JSAM(I) = 0
            ELSE IF(ISAM(I).GT.0) THEN
              ISAM(I) = 1
              JSAM(I) = 0
            ELSE
              JSAM(I) = 1
            ENDIF
          ELSE IF(ISWMDL(16).EQ.2) THEN
            KSAM(I) = MIN(KSAM(I),1)
          ELSE IF(ISWMDL(16).EQ.3) THEN
            IF(ISAM(I).GT.0) THEN
              ISAM(I) = 1
              JSAM(I) = 0
            ELSE
              JSAM(I) = 1
            ENDIF
          ENDIF
          IF(ISAM(I)+JSAM(I)+KSAM(I)+IDIR(I).EQ.0) JSAM(I) = 1
          IF(IDIR(I).GT.0) THEN
            IPAR(I) = 4
          ELSE IF(KSAM(I).GT.0) THEN
            IPAR(I) = 3
          ELSE IF(ISAM(I).GT.0) THEN
            IPAR(I) = 2
          ELSE
            IPAR(I) = 1
          ENDIF
C  mass fine correction
          IF(IPAR(I).EQ.1) THEN
            CALL MASSAD(IDPDG(I),IFL1P(I),IFL2P(I),PMASSD(1),
     &        XMASS(I),XMNEW,IRPDG(I),IRBAM(I))
            XMASS(I) = XMNEW
          ENDIF
C  debug output
          IF(IDEB(45).GE.15) WRITE(6,'(1X,A,/5X,I3,E12.4,4I5)')
     &      'PODIFF:DEBUG:IP,XMASS,ISAM,JSAM,KSAM,IDIR',
     &      IP,XMASS(I),ISAM(I),JSAM(I),KSAM(I),IDIR(I)
 130      CONTINUE
        ELSE
          WRITE(6,'(/1X,A,I3)') 'PODIFF:ERROR:INVALID IPAR',IPAR(I)
          CALL POABRT
        ENDIF
 100  CONTINUE
C
C  sampling of momentum transfer
      TT =  XSLOPE(TMIN,TMAX,XMASS(1),XMASS(2),IREJ)
      IF(IREJ.NE.0) THEN
        NSLP=NSLP+1
        IF(NSLP.LT.10) GOTO 50
        WRITE(6,'(1X,A,/10X,2I3,2E12.3)')
     &   'PODIFF:WARNING:TOO MANY SLOPE REJECTIONS:IPAR1,IPAR2,M1,M2',
     &   IPAR,XMASS
        CALL POABRT
      ENDIF
C  calculate new momenta in CMS
      CALL DIFKIN(XMASS(1),XMASS(2),TT,P1,P2,IREJ)
      IF(IREJ.NE.0) GOTO 50
      DO 120 I=1,4
        PP(I,1) = P1(I)
        PP(I,2) = P2(I)
 120  CONTINUE
C  debug output
      IF(IDEB(45).GE.5) THEN
        WRITE(6,'(1X,A,/5X,2I3,3E12.3)')
     &    'PODIFF:DEBUG:IPAR1,IPAR2,XMASS1,XMASS2,TT',IPAR,XMASS,TT
      ENDIF
C  actualize debug common
      IF(IMODE.EQ.1) THEN
        IDIFR1 = IPAR(1)
        IDIFR2 = IPAR(2)
      ENDIF
C  comment line for diffraction
      CALL REGPAR(30,IPROCE,IMODE,NPOSD(1),NPOSD(2),XMASS(1),XMASS(2),
     &   TT,ECMD,IPAR(1),IPAR(2),IDPDG(1),IDPDG(2),ICPOS,1)
C  write diffractive chains/particles
      DO 200 I=1,2
        I1 = I
        I2 = 3-I1
        PP(6,I1) = SIGN(PHEP(5,NPOSD(I1))**2,PHEP(5,NPOSD(I1)))
        PP(7,I1) = TT
        CALL PODREG(NPOSD(I1),NPOSD(I2),IFL1P(I1),IFL2P(I1),IPAR(I1),
     &    PP(1,I1),PP(1,I2),IPOSP(1,I1),IMODE,IREJ)
        IF(IREJ.NE.0) THEN
          IFAIL(7+I) = IFAIL(7+I)+1
          IF(IDEB(45).GE.3) WRITE(6,'(1X,A,2I3,E11.3)')
     &      'PODIFF:WARNING:REJECTION PODREG (I,IPAR,XM)',
     &      I,IPAR(I),XMASS(I)
          GOTO 50
        ENDIF
        ICOLOR(I1,ICPOS) = IPOSP(1,I1)
 200  CONTINUE
C  double-pomeron scattering?
      IF(IMODE.EQ.2) GOTO 150
C
C  diffractive final states
      DO 300 I=1,2
 110    CONTINUE
        IF(IPAR(I).EQ.0) THEN
C  vector meson production
          IF(IDPDG(I).EQ.22) THEN
            ISP = IPAMDL(3)
            IF(ISWMDL(21).GE.1) ISP = IPAMDL(4)
            CALL SDECAY(IPOSP(1,I),ISP,2)
          ENDIF
        ELSE
          IF(ISAM(I)+JSAM(I)+KSAM(I).EQ.0) JSAM(I) = 1
          IF(IDIR(I).GT.0) THEN
            IPAR(I) = 4
          ELSE IF(KSAM(I).GT.0) THEN
            IPAR(I) = 3
          ELSE IF(ISAM(I).GT.0) THEN
            IPAR(I) = 2
          ELSE
            IPAR(I) = 1
          ENDIF
          IPHIST(I,ICPOS) = IPAR(I)
C  update debug common
          KSPOM = ISAM(I)
          KSREG = JSAM(I)
          KHPOM = KSAM(I)
          KHDIR = IDIR(I)
          IDIFR1 = IPAR(1)
          IDIFR2 = IPAR(2)
          IF((IRPDG(I).NE.0).AND.(ISWMDL(23).GT.0)) THEN
C  resonance decay, pi+pi- background
            P1(1) = PHEP(1,IPOSP(1,I))+PHEP(1,IPOSP(2,I))
            P1(2) = PHEP(2,IPOSP(1,I))+PHEP(2,IPOSP(2,I))
            P1(3) = PHEP(3,IPOSP(1,I))+PHEP(3,IPOSP(2,I))
            P1(4) = PHEP(4,IPOSP(1,I))+PHEP(4,IPOSP(2,I))
            CALL REGPAR(1,IRPDG(I),IRBAM(I),IPOSP(1,I),IPOSP(2,I),
     &        P1(1),P1(2),P1(3),P1(4),0,0,0,0,IPOS,1)
            ISP = IPAMDL(3)
            IF((IDPDG(I).EQ.22).AND.(ISWMDL(21).EQ.2)) ISP = IPAMDL(4)
            CALL SDECAY(IPOS,ISP,2)
            IREJ = 0
          ELSE
C  particle-pomeron scattering
            NHEPS = NHEP
            CALL STDPAR(IPOSP(1,I),IPOSP(2,I),ISAM(I),JSAM(I),
     &        KSAM(I),IDIR(I),IREJ)
          ENDIF
        ENDIF
        IF(IREJ.NE.0) THEN
          IFAIL(20+I) = IFAIL(20+I)+1
          IF(IPAR(I).GT.1) THEN
            IF(IPAR(I).EQ.3) IFAIL(7+2*I) = IFAIL(7+2*I)+1
            IF(IPAR(I).EQ.4) IFAIL(8+2*I) = IFAIL(8+2*I)+1
            IF(IDIR(I).GT.0) THEN
              IDIR(I) = 0
            ELSE IF(KSAM(I).GT.0) THEN
              KSAM(I) = KSAM(I)-1
            ELSE IF(ISAM(I).GT.0) THEN
              ISAM(I) = ISAM(I)-1
            ENDIF
            NHEP = NHEPS
            JDAHEP(1,IPOSP(1,I)) = 0
            JDAHEP(2,IPOSP(1,I)) = 0
            JDAHEP(1,IPOSP(2,I)) = 0
            JDAHEP(2,IPOSP(2,I)) = 0
            GOTO 110
          ELSE
            IF(IDEB(45).GE.2) WRITE(6,'(1X,A,2I3,E11.3)')
     &        'PODIFF:WARNING:REJECTION STDPAR (I,IPAR,XM)',
     &        I,IPAR(I),XMASS(I)
            GOTO 50
          ENDIF
        ENDIF
 300  CONTINUE
C
*     IF(IPAR(1).GT.0) THEN
*       CALL FILHIS(XMASS(1),XMASS(1),IM1)
*       CALL ADDHIS(IM1)
*     ENDIF
*     IF(IPAR(2).GT.0) THEN
*       CALL FILHIS(XMASS(2),XMASS(2),IM2)
*       CALL ADDHIS(IM2)
*     ENDIF
      IDIF1 = IPAR(1)
      IDIF2 = IPAR(2)
C  update debug common
      KSPOM = KSPOMS+ISAM(1)+ISAM(2)
      KSREG = KSREGS+JSAM(1)+JSAM(2)
      KHPOM = KHPOMS+KSAM(1)+KSAM(2)
      KHDIR = KHDIRS+IDIR(1)+IDIR(2)
C
 150  CONTINUE
C  debug output
      IF(IDEB(45).GE.10) THEN
        WRITE(6,'(1X,A,/5X,7I4)')
     &   'PODIFF:DEBUG(2):IPAR1,IPAR2,IMOTH1,IMOTH2,MSOFT,MHARD,IMODE',
     &    IPAR,NPOSD,MSOFT,MHARD,IMODE
      ENDIF
      IF(IDEB(45).GE.15) THEN
        WRITE(6,'(2(/1X,A))') 'PODIFF:DEBUG:OUTPUT OF COMMON HEPEVS',
     &                        '===================================='
        CALL POPREV(0)
      ENDIF
      END
