C
C
      SUBROUTINE PODREG(IMOTH1,IMOTH2,IFL1,IFL2,IPAR,P1,P2,IPOSH,IMODE,
     &                  IREJ)
C*********************************************************************
C
C     perform chain construction of diffractive dissociation
C
C     input:     IMOTH1,2     index of mother particles in HEPEVS
C                IFL1,IFL2    particle numbers
C                             (IDPDG,IDBAM for quasi-elas. hadron)
C                IPAR         0  quasi-elasic scattering
C                             1  single chain configuration
C                             2  two chain configuration
C                P1           massive 4 momentum of first
C                P1(6)        virtuality/squ.mass of particle (GeV**2)
C                P1(7)        virtuality of Pomeron (neg, GeV**2)
C                P2           massive 4 momentum of second particle
C                IMODE        1   diffraction dissociation
C                             2   double-pomeron scattering
C
C     output:    IPOSH(1-2)   index of the particles in COMMON HEPEVS
C                IREJ         0  successful chain construction
C                             1  no chain construction possible
C
C*********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      DIMENSION IPOSH(2)
C
      PARAMETER(EPS  = 1.D-7,
     &          IONE = 1,
     &          ONE  = 1.D0,
     &          TWO  = 2.D0,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          DEPS = 1.D-10,
     &          ZERO = 0.D0)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
C  CM system of diffraction
      COMMON /DIFCMS/ ECMD,PCMD,PMASSD(2),PVIRTD(2),GAMBED(4),
     &                SIDD,CODD,SIFD,COFD,PDCMS(4,2),NPOSD(2)
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      DIMENSION P1(7),P2(7),PCH1(2,4)
C
      IREJ = 0
      ILTR1 = NHEP+1
C  elastic part
      IF(IPAR.EQ.0) THEN
        IF(IFL1.EQ.6666) THEN
C  pi+/pi- isotropic background
          CALL REGPAR(1,MPDGHA(IFL2),IFL2,IMOTH1,IMOTH2,P1(1),P1(2),
     &      P1(3),P1(4),0,0,0,0,IPOSH(1),1)
          CALL SDECAY(IPOSH(1),0,-2)
        ELSE
C  registration of resonance
          CALL REGPAR(1,IFL1,IFL2,IMOTH1,IMOTH2,P1(1),P1(2),P1(3),
     &      P1(4),0,0,0,0,IPOSH(1),1)
        ENDIF
C
C  diffractive dissociation
      ELSE IF((IPAR.GE.1).AND.(IPAR.LE.4)) THEN
C  calculation of resulting particle momenta
        IF(IMOTH1.EQ.NPOSD(1)) THEN
          K = 2
        ELSE
          K = 1
        ENDIF
        DO 100 I=1,4
          PCH1(2,I) = PDCMS(I,K)-P2(I)
          PCH1(1,I) = P1(I)-PCH1(2,I)
 100    CONTINUE
C  registration
        IF(IMODE.LT.2) CALL REGPAR(1,IFL1,IFL2,IMOTH1,IMOTH2,PCH1(1,1),
     &    PCH1(1,2),PCH1(1,3),PCH1(1,4),-1,0,ICA1,ICA2,IPOSH(1),1)
        CALL REGPAR(1,45,0,IMOTH2,IMOTH1,PCH1(2,1),PCH1(2,2),
     &              PCH1(2,3),PCH1(2,4),-1,0,ICB1,ICB2,IPOSH(2),1)
C  invalid IPAR
      ELSE
        WRITE(6,'(/1X,A,I6)') 'PODREG:ERROR:INVALID IPAR:',IPAR
        CALL POABRT
      ENDIF
C  back transformation
      CALL LTRHEP(ILTR1,NHEP,CODD,SIDD,COFD,SIFD,GAMBED(4),GAMBED(1),
     &            GAMBED(2),GAMBED(3))
      END
