C
C
      SUBROUTINE POHISR(IHARD,P1,P2,IPF1,IPF2,IPA1,IPA2,Q2H,XH1,XH2,
     &                  XHMAX1,XHMAX2,IPB1,IPB2,XISR1,XISR2,IREJ)
C********************************************************************
C
C     initial state radiation according to DGLAP evolution equations
C     (backward evolution, no spin effects)
C
C     input:    IHARD     index of hard Pomeron
C               P1,P2     4 momenta of hard scattered final partons
C                         (in CMS of hard scattering)
C               IPF1,2    flavours of final partons
C               IPA1,2    flavours of initial partons
C               Q2H       momentum transfer (squared, positive)
C               XH1,XH2   x values of initial partons
C               XHMAX1,2  max. x values allowed
C
C     output:   all emitted partons in COMMON /ABRISR/, final state
C               partons are the first two entries
C               shower evolution traced in COMMON /DGLAPI/
C               IPB1,2    flavours of new initial partons
C               XISR1,2   x values of new initial partons
C
C     attention: quark numbering according to PDG convention, but
C                0 for gluons
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      DIMENSION P1(4),P2(4)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  CM system of Pomeron/Reggeon exchange
      COMMON /POMCMS/ ECMP,PCMP,PMASSP(2),PVIRTP(2),GAMBEP(4),
     &                SIDP,CODP,SIFP,COFP,NPOSP(2),
     &                IDPDG1,IDBAM1,IDPDG2,IDBAM2
      COMMON /HADRON/ QMASS(6),BET,PCOUDI,PNORM,VALPRG(2),NFS
      CHARACTER*8 PDFNAM
      COMMON /PARPDF/ PDFLAM(2),PDFQ2M(2),PDFNAM(2),IPARID(2),PARVI(2),
     &                IPAVA(2),ITYPE(2),IGRP(2),ISET(2),IEXT(2),NPAOLD
C
      COMMON /HAQQAP/ AQQAL,AQQALI,AQQALF,AQQPD,
     &                NQQAL,NQQALI,NQQALF,NQQPD
      COMMON /DGLAPP/ Q2MISR(2),PMISR(2),ZMISR(2),AL2ISR(2),NFSISR
      COMMON /DGLAPI/ Q2SH(2,50),PT2SH(2,50),XPSH(2,50),ZPSH(2,50),
     &                THSH(2,50),SHAT(50),IFL1(2,50),IFL2(2,50),
     &                IBRA(2,100),IFANO(2),ISH(2),NACC
      COMMON /ABRISR/ IFLISR(2,150),PHISR(2,4,150),IPOISR(2,2,50)
C
      PARAMETER (ZERO   =  0.D0,
     &           IZERO  =  0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           RHOMAS =  0.766D0,
     &           PI     =  3.14159265359D0,
     &           DEPS   =  1.D-10,
     &           TINY   =  1.D-10)
C
      DIMENSION XHMA(2),NEXT(2),PD1(-6:6),PD2(-6:6),WGGAP(-6:6),
     &          WGPDF(-6:6),XHMI(2),GB(4),PM(4),PN(4),PC(2,4),Q2(2),
     &          IVAL(2),IPAL(2),IL(2),IFSUM(2),IDMO(2)
C
      IREJ = 0
      NTRY = 1000
      NITER = 0
C  debug output
      IF(IDEB(79).GE.10) THEN
        WRITE(6,'(1X,A,/1X,I10,3I3,5E11.3,2(/5X,4E12.3))')
     &    'POHISR:DEBUG:KEV,IHARD,IP1,IP2,Q2H,XH1,XH2,XHM1,XHM2:',
     &    KEVENT,IHARD,IPA1,IPA2,Q2H,XH1,XH2,XHMAX1,XHMAX2,P1,P2
      ENDIF
      IF(IHARD.EQ.0) RETURN
C
 10   CONTINUE
      NACC = 0
      IDMO(1) = IDPDG1
      IDMO(2) = IDPDG2
C
C  copy final state partons to local fields
      DO 50 K=1,2
        IF(IHARD.EQ.1) THEN
          IPOISR(K,1,1) = 1
        ELSE
          IPOISR(K,1,IHARD) = IPOISR(K,2,IHARD-1)+1
        ENDIF
        IPAL(K) = IPOISR(K,1,IHARD)
 50   CONTINUE
      DO 55 I=1,4
        PHISR(1,I,IPAL(1)) = P1(I)
        PHISR(2,I,IPAL(2)) = P2(I)
 55   CONTINUE
      IFLISR(1,IPAL(1)) = IPF1
      IFLISR(2,IPAL(2)) = IPF2
C
C  check limitations, initialize COMMON /DGLAPI/
      IF((Q2H.GT.Q2MISR(1)).AND.(XH1.LT.XHMAX1)) THEN
        NEXT(1) = 1
        Q2SH(1,1) = Q2H
      ELSE
        NEXT(1) = 0
        Q2SH(1,1) = ZERO
      ENDIF
      IF((Q2H.GT.Q2MISR(2)).AND.(XH2.LT.XHMAX2)) THEN
        NEXT(2) = 1
        Q2SH(2,1) = Q2H
      ELSE
        NEXT(2) = 0
        Q2SH(2,1) = ZERO
      ENDIF
C
      ISH(1) = 1
      ISH(2) = 1
      XPSH(1,1) = XH1
      XPSH(2,1) = XH2
C
      IFL1(1,1) = IPA1
      IVAL(1) = 0
      IF((IPA1.EQ.22).OR.(IPA1.EQ.45)) THEN
        NEXT(1) = 0
      ELSE IF(ABS(IPA1).GT.6) THEN
        IFL1(1,1) = SIGN(ABS(IPA1)-6,IPA1)
        IVAL(1) = 1
      ENDIF
      IFL1(2,1) = IPA2
      IVAL(2) = 0
      IF((IPA2.EQ.22).OR.(IPA2.EQ.45)) THEN
        NEXT(2) = 0
      ELSE IF(ABS(IPA2).GT.6) THEN
        IFL1(2,1) = SIGN(ABS(IPA2)-6,IPA2)
        IVAL(2) = 1
      ENDIF
C
      IF(IDEB(79).GE.17) WRITE(6,'(1X,A,/5X,2I2,3E12.3)')
     &  'POHISR:INITIAL TESTS (NEXT1,2 Q2H Q21,2)',NEXT,Q2H,Q2MISR
      IF(NEXT(1)+NEXT(2).EQ.0) GOTO 800
C
C  initialize parton shower loop
      B0QCD = (33.D0-TWO*NFSISR)/6.D0
      AL2ISR(1) = PDFLAM(1)
      AL2ISR(2) = PDFLAM(2)
      XHMA(1) = XHMAX1
      XHMA(2) = XHMAX2
      XHMI(1) = PMISR(1)/PCMP
      XHMI(2) = PMISR(2)/PCMP
      ZPSH(1,1) = ONE
      ZPSH(2,1) = ONE
      SHAT1 = XH1*XH2*ECMP**2
      IF(IPAMDL(109).EQ.1) THEN
        PT2SH(1,1) = Q2H
      ELSE
        PT2SH(1,1) = Q2H*(ONE-Q2H/SHAT1)
      ENDIF
      PT2SH(2,1) = PT2SH(1,1)
      IF(PT2SH(1,1).LT.Q2MISR(1)) NEXT(1) = 0
      IF(PT2SH(2,1).LT.Q2MISR(2)) NEXT(2) = 0
      THSH(1,1) = 2.D0*SQRT(Q2H/SHAT1)
      THSH(2,1) = THSH(1,1)
      IFANO(1) = 0
      IFANO(2) = 0
      ZZ = ONE
      IF(IREJ.NE.0) GOTO 800
C
C  main generation loop
C -------------------------------------------------
 100  CONTINUE
C  choose parton side to become solved
        IF((NEXT(1)+NEXT(2)).EQ.2) THEN
          IF(Q2SH(1,ISH(1)).GT.Q2SH(2,ISH(2))) THEN
            IP = 1
          ELSE IF(Q2SH(2,ISH(2)).GT.Q2SH(1,ISH(1))) THEN
            IP = 2
          ELSE
            IP = MAX(INT(RNDM(SHAT1)*2.D0+0.999999D0),1)
          ENDIF
        ELSE IF(NEXT(1).EQ.1) THEN
          IP = 1
        ELSE IF(NEXT(2).EQ.1) THEN
          IP = 2
        ELSE
          GOTO 800
        ENDIF
        INDX = ISH(IP)
C  INDX now parton position of parton to become solved
C  IP   now side to be treated
        XP = XPSH(IP,INDX)
        Q2P = Q2SH(IP,INDX)
        PT2 = PT2SH(IP,INDX)
        IFLB = IFL1(IP,INDX)
C  check available x
        XMIP = XHMI(IP)
C  cutoff by x limitation: no further development
        IF((XHMA(IP)-XP).LT.XMIP*TWO) THEN
          NEXT(IP) = 0
          Q2SH(IP,INDX) = ZERO
          IF(IDEB(79).GE.17) THEN
            WRITE(6,'(1X,A,/5X,3E12.4,2I3)')
     &        'POHISR:DEBUG:EVOLUTION X-STOP (XP,XMIP,XHMA,IP,INDX)',
     &        XP,XMIP,XHMA(IP),IP,INDX
          ENDIF
          GOTO 100
        ENDIF
C  initial value of evolution variable t
        TT = LOG(AQQALI*Q2P/AL2ISR(IP))
        DO 110 I=-NFSISR,NFSISR
          WGGAP(I) = ZERO
          WGPDF(I) = ZERO
 110    CONTINUE
C  DGLAP weights
        ZMIN = XP/XHMA(IP)
        ZMAX = XP/(XP+XMIP)
        CF = 4./3.
C  q --> q g, g --> g g
        IF(IFLB.EQ.0) THEN
          WGGAP(0) = CF*((ZMAX**2-ZMIN**2)/TWO-TWO*(ZMAX-ZMIN)
     &      +TWO*LOG(ZMAX/ZMIN))
          DO 120 I=1,NFSISR
            WGGAP(I)  = WGGAP(0)
            WGGAP(-I) = WGGAP(0)
 120      CONTINUE
          WGGAP(0) = 6.D0*((ZMIN**3-ZMAX**3)/3.D0+(ZMAX**2-ZMIN**2)/TWO
     &      -TWO*(ZMAX-ZMIN)+LOG(ZMAX/ZMIN*(ONE-ZMIN)/(ONE-ZMAX)))
C  q --> g q, g --> q qb
        ELSE IF(ABS(IFLB).LE.6) THEN
          WGGAP(IFLB) = CF*((ZMIN**2-ZMAX**2)/TWO-ZMAX+ZMIN
     &      -TWO*LOG((ONE-ZMAX)/(ONE-ZMIN)))
          IF(IVAL(IP).EQ.0) WGGAP(0) = 0.5D0*(2./3.*(ZMAX**3-ZMIN**3)
     &      -ZMAX**2+ZMIN**2+ZMAX-ZMIN)
        ELSE
          WRITE(6,'(/1X,A,I7)') 'POHISR:ERROR:UNSUPPORTED PART.ID',IFLB
          CALL POABRT
        ENDIF
C
C  rejection loop for z,t sampling
C ------------------------------------
 200    CONTINUE
          NITER = NITER+1
          IF(NITER.GE.NTRY) THEN
            WRITE(6,'(1X,A,2I6)') 'POHISR:WARNING:TOO MANY REJECTIONS',
     &        NITER,NTRY
            CALL POPREV(-1)
C  clean up event
            IREJ = 1
            GOTO 10
          ENDIF
C  PDF weights
          IF(NQQALI.EQ.1) THEN
            SCALE2 = PT2*AQQPD
          ELSE
            SCALE2 = Q2P*AQQPD
          ENDIF
          CALL PDIS(XP,SCALE2,IP,PD1)
C  anomalous/resolved evolution
          IF(IPAMDL(110).GE.1) THEN
            IF((IDMO(IP).EQ.22).AND.(IFLB.NE.0)
     &         .AND.(IFLB.NE.21)) THEN
              WGDIR = ZERO
              CALL POPDFB(IFLB,XP,SCALE2,PVIRTP(IP),WGDIR)
              XI = RNDM(XP)*PD1(IFLB)
              IF(WGDIR.GT.XI) THEN
C  debug output
                IF(IDEB(79).GE.17) WRITE(6,'(1X,A,/5X,4E12.5,I2,I3)')
     &       'POHISR:DEBUG:ANOM.TERM (WGDIR,WGPDF,X,SCALE2,IP,IFLB)',
     &            WGDIR,PD1(IFLB),XP,SCALE2,IP,IFLB
                Q2SH(IP,INDX) = ZERO
                NEXT(IP) = 0
                IFANO(IP) = INDX
                GOTO 100
              ENDIF
            ENDIF
          ENDIF
C
          WGTOT = ZERO
          DO 210 I=-NFSISR,NFSISR
            WGPDF(I) = PD1(I)/PD1(IFLB)*5.D0
            WGTOT = WGTOT+WGPDF(I)*WGGAP(I)
 210      CONTINUE
C
 215      CONTINUE
C  sample new t value
          TT = TT*EXP(MAX(-10.D0,LOG(RNDM(SHAT1))*B0QCD/WGTOT))
          Q2NEW = AL2ISR(IP)*EXP(TT)/AQQALI
C  debug output
          IF(IDEB(79).GE.20) WRITE(6,'(1X,A,E12.5)')
     &      'POHISR:DEBUG:PRE-SELECTED Q2:',Q2NEW
C  compare to limits
          IF(Q2NEW.LT.Q2MISR(IP)) THEN
            Q2SH(IP,INDX) = ZERO
            NEXT(IP) = 0
            IF(IDEB(79).GE.17) WRITE(6,'(1X,A,2E10.3,2I3)')
     &        'POHISR:DEBUG:EVOLUTION Q2-STOP (Q2,Q2MIN,IP,INDX):',
     &        Q2NEW,Q2MISR(IP),IP,INDX
            GOTO 100
          ENDIF
          Q2SH(IP,INDX) = Q2NEW
          TT = LOG(AQQALI*Q2NEW/AL2ISR(IP))
C  selection of flavours
          XI = WGTOT*RNDM(TT)
          IFLA = -NFSISR-1
 220      CONTINUE
            IFLA = IFLA+1
            XI = XI-WGPDF(IFLA)*WGGAP(IFLA)
          IF((XI.GT.ZERO).AND.(IFLA.LT.NFSISR)) GOTO 220
C  debug output
          IF(IDEB(79).GE.20) WRITE(6,'(1X,A,2I3)')
     &      'POHISR:DEBUG:PRE-SELECTED IFLA (IFLA,IFLB):',IFLA,IFLB
C  selection of z
          CALL POHZSP(IFLA,IFLB,NFSISR,ZMIN,ZMAX,ZZ)
C  debug output
          IF(IDEB(79).GE.20) WRITE(6,'(1X,A,E12.3)')
     &      'POHISR:DEBUG:PRE-SELECTED ZZ',ZZ
C  angular ordering
          THETA = 4.D0*ZZ**2*Q2NEW/((ECMP*XP)**2*(ONE-ZZ))
          IF(THETA.GT.THSH(IP,INDX)) THEN
            IF(IDEB(79).GE.20) WRITE(6,'(1X,A,2E12.3)')
     &        'POHISR:DEBUG:REJECT BY ANGLE (NEW/OLD)',
     &        THETA,THSH(IP,INDX)
            GOTO 215
          ENDIF
C  rejection weight given by new PDFs
          XNEW = XP/ZZ
          PT2NEW = Q2NEW*(ONE-ZZ)
          IF(NQQALI.EQ.1) THEN
            SCALE2 = PT2NEW*AQQPD
          ELSE
            SCALE2 = Q2NEW*AQQPD
          ENDIF
          IF(SCALE2.LT.Q2MISR(IP)) THEN
            Q2SH(IP,INDX) = ZERO
            NEXT(IP) = 0
            IF(IDEB(79).GE.17) WRITE(6,'(1X,A,2E10.3,2I3)')
     &        'POHISR:DEBUG:EVOL.Q2-STOP (SCALE2,Q2MIN,IP,INDX):',
     &        Q2NEW,Q2MISR(IP),IP,INDX
            GOTO 100
          ENDIF
          CALL PDIS(XNEW,SCALE2,IP,PD2)
          IF(PD2(IFLA).LT.1.D-10) GOTO 200
          CALL PDIS(XP,SCALE2,IP,PD1)
          PD1(IFLB) = MAX(PD1(IFLB),1.D-10)
          WGF = PD2(IFLA)/PD1(IFLB)/WGPDF(IFLA)
          IF(NQQALI.EQ.1) WGF = WGF*LOG(Q2NEW*AQQALI/AL2ISR(IP))
     &      /LOG(PT2NEW*AQQALI/AL2ISR(IP))
          IF((WGF.GT.ONE).AND.(IDEB(79).GE.2)) THEN
            WRITE(6,'(1X,A,E12.3)') 'POHISR:WARNING:FINAL WEIGHT:',WGF
            WRITE(6,'(6X,A,I7,2I3,3E11.3)')
     &      'EV,IFLA,IFLB,Q2,PT2,Z:',KEVENT,IFLA,IFLB,Q2NEW,PT2NEW,ZZ
          ENDIF
        IF(WGF.LT.RNDM(XNEW)) GOTO 200
C
        IF(IDEB(79).GE.15) THEN
          WRITE(6,'(1X,A,/3X,3I3,3E11.3)')
     &      'POHISR:DEBUG:ACCEPTED (IP,IFLA,IFLB,PT2,Q2,Z)',
     &      IP,IFLA,IFLB,PT2NEW,Q2NEW,ZZ
        ENDIF
C
C  branching accepted, registration
        Q2SH(IP,INDX) = Q2NEW
        PT2SH(IP,INDX) = PT2NEW
        ZPSH(IP,INDX) = ZZ
        IFL2(IP,INDX) = IFLA-IFLB
        Q2SH(IP,INDX+1) = Q2NEW
        PT2SH(IP,INDX+1) = PT2SH(IP,INDX)
        XPSH(IP,INDX+1) = XNEW
        THSH(IP,INDX+1) = THETA
        IFL1(IP,INDX+1) = IFLA
        ISH(IP) = ISH(IP)+1
        NACC = NACC+1
        SHAT(NACC) = SHAT1
        IBRA(1,NACC) = IP
        IBRA(2,NACC) = INDX
        SHAT1 = SHAT1/ZZ
C
C  generation of next branching
      IF(NEXT(1)+NEXT(2).NE.0) GOTO 100
C
 800  CONTINUE
      IPB1 = SIGN(ABS(IFL1(1,ISH(1)))+6*IVAL(1),IFL1(1,ISH(1)))
      IPB2 = SIGN(ABS(IFL1(2,ISH(2)))+6*IVAL(2),IFL1(2,ISH(2)))
      XISR1 = XPSH(1,ISH(1))
      XISR2 = XPSH(2,ISH(2))
C  parton kinematics
      IF(NACC.GT.0) THEN
C  final partons in CMS
        PM(3) = (XH1-XH2)*ECMP/TWO
        PM(4) = (XH1+XH2)*ECMP/TWO
        SH = XH1*XH2*ECMP**2
        SSH = SQRT(SH)
        GB(3) = PM(3)/SSH
        GB(4) = PM(4)/SSH
        CALL ALTRA(GB(4),ZERO,ZERO,-GB(3),P1(1),P1(2),P1(3),
     &    P1(4),PTOT1,PHISR(1,1,IPAL(1)),PHISR(1,2,IPAL(1)),
     &    PHISR(1,3,IPAL(1)),PHISR(1,4,IPAL(1)))
        CALL ALTRA(GB(4),ZERO,ZERO,-GB(3),P2(1),P2(2),P2(3),
     &    P2(4),PTOT1,PHISR(2,1,IPAL(2)),PHISR(2,2,IPAL(2)),
     &    PHISR(2,3,IPAL(2)),PHISR(2,4,IPAL(2)))
        IL(1) = 1
        IL(2) = 1
        DO 900 I=1,NACC
          IPA = IBRA(1,I)
          IPB = 3-IPA
          IL(IPA) = IBRA(2,I)
C  initial partons in CMS
          SH = SHAT(I)
          SSH = SQRT(SH)
          SHZ = SH/ZPSH(IPA,IL(IPA))
          SSHZ = SQRT(SHZ)
          Q2(1) = Q2SH(1,IL(1))
          Q2(2) = Q2SH(2,IL(2))
          PC(1,1) = ZERO
          PC(1,2) = ZERO
          PC(1,3) = SQRT((SH+Q2(1)+Q2(2))**2-4.D0*Q2(1)*Q2(2))/(TWO*SSH)
          PC(1,4) = (SH-Q2(1)+Q2(2))/(TWO*SSH)
          PC(2,1) = ZERO
          PC(2,2) = ZERO
          PC(2,3) = -PC(1,3)
          PC(2,4) = SSH-PC(1,4)
C  emitted parton (on mass shell)
          XMS4 = POPAMA(IFL2(IPA,IL(IPA)),1)**2
          EE3 = (SHZ-Q2(IPA)+Q2(IPB)-XMS4)/(TWO*SSH)
          PZ3 = (TWO*PC(IPA,4)*EE3+Q2(IPA)+Q2SH(IPA,IL(IPA)+1)+XMS4)
     &          /(TWO*PC(IPA,3))
          PT3 = EE3**2-PZ3**2+Q2SH(IPA,IL(IPA)+1)
          IF(PT3.LT.ZERO) THEN
            IF(IDEB(79).GE.5) WRITE(6,'(1X,A,E12.3)')
     &        'POHISR:WARNING:REJECTION BY PT3',PT3
            GOTO 10
          ENDIF
          PT3 = SQRT(PT3)
          CALL SFECFE(SFE,CFE)
          PX3 = CFE*PT3
          PY3 = SFE*PT3
          IPAL(IPA) = IPAL(IPA)+1
          PHISR(IPA,1,IPAL(IPA)) = PX3
          PHISR(IPA,2,IPAL(IPA)) = PY3
          PHISR(IPA,3,IPAL(IPA)) = PZ3-PC(IPA,3)
          PHISR(IPA,4,IPAL(IPA)) = EE3-PC(IPA,4)
          IFLISR(IPA,IPAL(IPA)) = IFL2(IPA,IL(IPA))
          PC(IPA,1) = PX3
          PC(IPA,2) = PY3
          PC(IPA,3) = PZ3
          PC(IPA,4) = EE3
C  boost into new CMS
          DO 842 K=1,4
            GB(K) = (PC(1,K)+PC(2,K))/SSHZ
 842      CONTINUE
          CALL ALTRA(GB(4),-GB(1),-GB(2),-GB(3),PC(1,1),PC(1,2),
     &      PC(1,3),PC(1,4),PTOT1,PM(1),PM(2),PM(3),PM(4))
          COG= PM(3)/PTOT1
          SIG= SQRT((ONE-COG)*(ONE+COG))
          COH=ONE
          SIH=ZERO
          IF(PTOT1*SIG.GT.DEPS) THEN
            COH=PM(1)/(SIG*PTOT1)
            SIH=PM(2)/(SIG*PTOT1)
            ANORF=SQRT(COH*COH+SIH*SIH)
            COH=COH/ANORF
            SIH=SIH/ANORF
          ENDIF
          PX3 = ZERO
          PY3 = ZERO
          PZ3 = ZERO
          DO 845 K=1,2
            DO 844 L=IPOISR(K,1,IHARD),IPAL(K)
              CALL ALTRA(GB(4),-GB(1),-GB(2),-GB(3),
     &          PHISR(K,1,L),PHISR(K,2,L),PHISR(K,3,L),PHISR(K,4,L),
     &          PTOT1,PM(1),PM(2),PM(3),PM(4))
              CALL TRANI(PM(1),PM(2),PM(3),COG,SIG,COH,SIH,PN(1),
     &          PN(2),PN(3))
              CALL TRANS(PN(1),PN(2),PN(3),ONE,ZERO,COH,SIH,
     &          PHISR(K,1,L),PHISR(K,2,L),PHISR(K,3,L))
              PHISR(K,4,L) = PM(4)
              PX3 = PX3+PHISR(K,1,L)
              PY3 = PY3+PHISR(K,2,L)
              PZ3 = PZ3+PHISR(K,3,L)
 844        CONTINUE
 845      CONTINUE
 900    CONTINUE
C  boost to global CMS
        PM(3) = (XISR1-XISR2)/TWO
        PM(4) = (XISR1+XISR2)/TWO
        SSH = SQRT(XISR1*XISR2)
        GB(3) = PM(3)/SSH
        GB(4) = PM(4)/SSH
        DO 945 K=1,2
          DO 944 L=IPOISR(K,1,IHARD),IPAL(K)
            CALL ALTRA(GB(4),ZERO,ZERO,GB(3),PHISR(K,1,L),PHISR(K,2,L),
     &        PHISR(K,3,L),PHISR(K,4,L),PTOT1,PM(1),PM(2),PM(3),PM(4))
            PHISR(K,1,L) = PM(1)
            PHISR(K,2,L) = PM(2)
            PHISR(K,3,L) = PM(3)
            PHISR(K,4,L) = PM(4)
 944      CONTINUE
 945    CONTINUE
      ENDIF
      IPOISR(1,2,IHARD) = IPAL(1)
      IPOISR(2,2,IHARD) = IPAL(2)
C  debug output
      IF(IDEB(79).GE.10) THEN
        WRITE(6,'(1X,A,2I5,/6X,A)')
     &    'POHISR:DEBUG:ISR CONFIGURATION(NITER,NACC)',NITER,NACC,
     &    ' SIDE   NO.   IFLB IFLC     Q2SH    PT2SH     XH         ZZ'
        DO 600 II=1,NACC
          K = IBRA(1,II)
          I = IBRA(2,II)
          WRITE(6,'(5X,4I5,4E11.3)')
     &      K,I,IFL1(K,I),IFL2(K,I),Q2SH(K,I),PT2SH(K,I),XPSH(K,I),
     &      ZPSH(K,I)
 600    CONTINUE
        WRITE(6,'(1X,A,2I10/,6X,A,2E12.3,2I5)') 'NUMBER OF EMISSIONS',
     &    ISH(1)-1,ISH(2)-1,'NEW X1,X2,IFL1,ILF2',XISR1,XISR2,IPB1,IPB2
C  check of final configuration
        PX3 = ZERO
        PY3 = ZERO
        PZ3 = ZERO
        EE3 = ZERO
        IFSUM(1) = 0
        IFSUM(2) = 0
        WRITE(6,'(1X,A)') 'POHISR:DEBUG:OUTGOING PARTONS'
        DO 745 K=1,2
          DO 744 L=IPOISR(K,1,IHARD),IPOISR(K,2,IHARD)
            WRITE(6,'(6X,2I4,I6,4E11.3)') K,L,IFLISR(K,L),
     &        PHISR(K,1,L),PHISR(K,2,L),PHISR(K,3,L),PHISR(K,4,L)
            IFSUM(K) = IFSUM(K)+ IFLISR(K,L)
            PX3 = PX3 + PHISR(K,1,L)
            PY3 = PY3 + PHISR(K,2,L)
            PZ3 = PZ3 + PHISR(K,3,L)
            EE3 = EE3 + PHISR(K,4,L)
 744      CONTINUE
 745    CONTINUE
        IFSUM(1) = IFSUM(1)-IPB1
        IFSUM(2) = IFSUM(2)-IPB2
        PZ3 = PZ3 -(XISR1-XISR2)*ECMP/TWO
        EE3 = EE3 -(XISR1+XISR2)*ECMP/TWO
        WRITE(6,'(1X,A,2I4,4E11.3)') 'CHECK:IFL1,2 PCM(1-4)',
     &    IFSUM,PX3,PY3,PZ3,EE3
      ENDIF
      END
