C
C
      SUBROUTINE SELPT(EE,PTLOW,PTHIGH,PTS,BETA,IMODE)
C***********************************************************************
C
C    select pt from different distributions
C
C    input:    EE            energy (for initialization only)
C                            otherwise x value of corresponding parton
C              PTLOW         lower pt limit
C              PTHIGH        upper pt limit
C                            (PTHIGH > 20 will cause DEXP underflows)
C
C              IMODE = 0     dNs/dP_t = P_t * ASS * exp(-BETA*P_t**2)
C              IMODE = 1     dNs/dP_t = P_t * ASS * exp(-BETA*P_t)
C              IMODE = 2     dNs/dP_t according photon wave function
C              IMODE = 10    no sampling
C
C              IMODE = -100+IMODE    initialization according to
C                                    given limitations
C
C    output:   PTS           sampled pt value
C    initialization:
C              BETA          soft pt slope in central region
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           PI2    =  6.28318530718D0,
     &           OHALF  =  0.5D0,
     &           AMIN   =  1.D-2,
     &           EPS    =  1.D-7,
     &           DEPS   =  1.D-30)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  CM system of Pomeron/Reggeon exchange
      COMMON /POMCMS/ ECMP,PCMP,PMASSP(2),PVIRTP(2),GAMBEP(4),
     &                SIDP,CODP,SIFP,COFP,NPOSP(2),
     &                IDPDG1,IDBAM1,IDPDG2,IDBAM2
C
      COMMON /AVKLMN/ AVERI,AVERK,AVERL,AVERM,AVERN
      COMMON /XSECPT/ SIGS,DSIGHP,SIGH,FS,FH,BETAS(3),AAS,PTCON
      DOUBLE PRECISION CONN0,CONN1
      EXTERNAL CONN0,CONN1
C
C  initialization
      IF(IMODE.LT.0) GOTO 100
C
C  sampling of pt values according to IMODE
      PX = PTHIGH
      PTS = ZERO
      IF(PX.LT.AMIN) RETURN
C
      IF((PX-PTLOW).LT.0.01) THEN
        IF(IDEB(5).GE.3) WRITE(6,'(1X,A,2E12.3,I3)')
     &    'SELPT:WARNING:PTLOW,PTHIGH,IMODE ',PTLOW,PTHIGH,IMODE
        RETURN
      ENDIF
      IF(IMODE.EQ.0) THEN
 25     CONTINUE
          XI1 = RNDM(PX)*(ONE-EXP(-BETA*PX**2)) + EXP(-BETA*PX**2)
          PTS = SQRT(-ONE/BETA*LOG(XI1))
        IF((PTS.GT.PTHIGH).OR.(PTS.LT.PTLOW)) GOTO 25
      ELSE IF(IMODE.EQ.1) THEN
        XIMIN = EXP(-BETA*PTHIGH)
        XIDEL = ONE-XIMIN
 50     CONTINUE
          PTS = -LOG((XIDEL*RNDM(XIDEL)+XIMIN)
     &              *(XIDEL*RNDM(XIMIN)+XIMIN)+DEPS)/BETA
        IF(PTS.LT.XMT) GOTO 50
        PTS = SQRT(PTS**2-XMT2)
        IF((PTS.GT.PTHIGH).OR.(PTS.LT.PTLOW)) GOTO 50
      ELSE IF(IMODE.EQ.2) THEN
        IF(EE.GE.ZERO) THEN
          P2 = PVIRTP(1)
        ELSE
          P2 = PVIRTP(2)
        ENDIF
        XV = ABS(EE)
        AA = (ONE-XV)*XV*P2+PARMDL(25)
 75     CONTINUE
          PTS = SQRT(AA/(RNDM(PX)+EPS)-AA)
        IF((PTS.GT.PTHIGH).OR.(PTS.LT.PTLOW)) GOTO 75
      ELSE IF(IMODE.NE.10) THEN
        WRITE(6,'(/1X,A,I4)') 'SELPT:ERROR:UNSUPPORTED DISTRIBUTION',
     &    IMODE
        CALL POABRT
      ENDIF
C  debug output
      IF(IDEB(5).GE.20) THEN
        WRITE(6,'(1X,A,I3,4E10.3)')
     &    'SELPT:DEBUG:MODE,BET,PTMI,PTMA,PT',
     &    IMODE,BETA,PTLOW,PTHIGH,PTS
      ENDIF
      RETURN
C
C  initialization
 100  CONTINUE
        PTSMIN = PTLOW
        PTSMAX = PTHIGH
        PTCON = PTHIGH
C  calculation of parameters
        INIT = IMODE+100
        AAS = ZERO
        IF(INIT.EQ.0) THEN
          BETAS(1) = PARMDL(23)+0.15D0*LOG(EE)*(PARMDL(24)-PARMDL(23))
          IF(SIGH/SIGS.LT.0.0001D0) THEN
            AAS = ZERO
            BETA = BETAS(1)
          ELSE
            XTOL = 1.D-4
            METHOD = 1
            MAXF = 500
            BETUP = 2.5D0
            BETLO = -2.D0
            BETA = 0.D0
            BETA = DZEROX(BETLO,BETUP,XTOL,MAXF,CONN0,METHOD)
            IF(BETA.LT.EPS) THEN
              BETA = 0.01
            ENDIF
          ENDIF
        ELSE IF(INIT.EQ.1) THEN
          XMT = PARMDL(43)
          XMT2 = XMT*XMT
          BETAS(1) = PARMDL(21)+0.15D0*LOG(EE)*(PARMDL(22)-PARMDL(21))
          IF(SIGH/SIGS.LT.0.0001D0) THEN
            AAS = ZERO
            BETA = BETAS(1)
          ELSE
            XTOL = 1.D-4
            METHOD = 1
            MAXF = 500
            BETUP = 7.D0
            BETLO = -3.D0
            BETA = 0.D0
            BETA = DZEROX(BETLO,BETUP,XTOL,MAXF,CONN1,METHOD)
            IF(BETA.LT.EPS) THEN
              BETA = 0.01
            ENDIF
          ENDIF
        ELSE IF(INIT.EQ.10) THEN
          IF(IDEB(5).GT.10)
     &      WRITE(6,'(/1X,A)') 'SELPT:WARNING:NO SOFT PT SAMPLING'
          RETURN
        ELSE
          WRITE(6,'(1X,A,I4)') 'SELPT:ERROR:UNSUPPORTED DISTRIBUTION',
     &      INIT
          CALL POABRT
        ENDIF
        BETA = MIN(BETA,BETAS(1))
C
        IF(BETA.LE.ZERO) THEN
          WRITE(6,'(1X,A,2E12.3)')
     &      'SELPT:WARNING:PARAMETER BETA,AAS NEG.',BETA,AAS
          WRITE(6,'(1X,A,4E12.3)') ' SIGS,DSIGHP,SIGH,PTCON:',
     &      SIGS,DSIGHP,SIGH,PTCON
          CALL POPREV(-1)
        ENDIF
C  output of initialization parameters
        IF(IDEB(5).GE.10) THEN
          WRITE(6,'(1X,A,I3)') 'SELPT:DEBUG:INITIALIZATION,MODEL',INIT
          WRITE(6,'(5X,A,2E15.3)') 'BETA, AAS       ',BETA,AAS
          WRITE(6,'(5X,A,3E15.3)') 'ECM,PTMIN,PTMAX ',EE,PTSMIN,PTSMAX
          WRITE(6,'(5X,A,3E15.3)') 'SIGS,DSIGHP,SIGH',SIGS,DSIGHP,SIGH
        ENDIF
      END
