C
C
      SUBROUTINE VALFLA(IPAR,IFL1,IFL2,XMASS)
C**********************************************************************
C
C     selection of valence falvour decomposition of particle IPAR
C
C     input:    IPAR   particle index in common HEPEVS
C                      -1   initialization
C                      -2   output of statistics
C               XMASS  mass of particle
C                      (essential for Pomeron:
C                       mass dependent flavour sampling)
C
C     output:   IFL1,IFL2
C               baryon: IFL1  diquark flavour
C               (valence flavours according to PDG conventions)
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           THREE  =  3.D0,
     &           OHALF  =  0.5D0,
     &           EPS    =  0.1D0,
     &           DEPS   =  1.D-40)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      IF(IPAR.GT.0) THEN
        K = IPAR
C  select particle code
        ID1 = IDHEP(K)
        ID2 = IMPART(K)
        IF(IPOBA3(K,2).EQ.0) THEN
C  photon
          IF(ID1.EQ.22) THEN
C  charge dependent flavour sampling
            K = INT(RNDM(XMASS)*6.D0+1.D0)
            IF(K.LE.4) THEN
              IFL1 = 2
              IFL2 = -2
            ELSE IF(K.EQ.5) THEN
              IFL1 = 1
              IFL2 = -1
            ELSE
              IFL1 = 3
              IFL2 = -3
            ENDIF
            IF(RNDM(XI).LT.0.5D0) THEN
              K = IFL1
              IFL1 = IFL2
              IFL2 = K
            ENDIF
C  Pomeron
          ELSE IF(ID1.EQ.45) THEN
            IF(ISWMDL(19).EQ.0) THEN
C  SU(3) symmetric valences
              K = INT(RNDM(XMASS)*3.D0+1.D0)
              IF(RNDM(XI).LT.OHALF) THEN
                IFL1 = K
              ELSE
                IFL1 = -K
              ENDIF
              IFL2 = -IFL1
            ELSE IF(ISWMDL(19).EQ.1) THEN
C  mass dependent flavour sampling
              CALL SEAFLA(IPAR,IFL1,IFL2,XMASS)
            ELSE
              WRITE(6,'(/1X,A,I5)') 'VALFLA:ERROR:INVALID ISWMDL(19)',
     &          ISWMDL(19)
              CALL POABRT
            ENDIF
C  meson
          ELSE
            K = INT(2.D0*RNDM(XMASS)+1.D0)
            K = MIN(K,2)
            IFL1 = IPDGQU(K,ID2)
            K = MOD(K,2) + 1
            IFL2 = IPDGQU(K,ID2)
            IF(IFL1.EQ.0) CALL SEAFLA(IPAR,IFL1,IFL2,XMASS)
          ENDIF
C  baryon
        ELSE
          K = INT(3.D0*RNDM(XMASS)+1.000001D0)
          K = MIN(K,3)
          K1 = MOD(K,3)+1
          K2 = MOD(K1,3)+1
          K3 = NDIQU(ABS(IPDGQU(K1,ID2)),ABS(IPDGQU(K2,ID2)))
          IFL1 = SIGN(K3,ID1)
          IFL2 = IPDGQU(K,ID2)
        ENDIF
C  debug output
        IF(IDEB(46).GE.10) THEN
        WRITE(6,'(1X,A,I5,E12.3,2I7)') 'VALFLA:DEBUG:IPAR,MASS,FL1,2',
     &    IPAR,XMASS,IFL1,IFL2
        ENDIF
      ELSE IF(IPAR.EQ.-1) THEN
C  initialization
      ELSE IF(IPAR.EQ.-2) THEN
C  output of final statistics
      ELSE
        WRITE(6,'(1X,A,I10)') 'VALFLA:ERROR:INVALID IPAR',IPAR
        CALL POABRT
      ENDIF
      END
